# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from aioketraapi.api_client import ApiClient
from aioketraapi.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class KeypadOperationsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def activate_button_post(self, keypad_name, button_name, level, **kwargs):  # noqa: E501
        """Activate a button  # noqa: E501

        Activates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.activate_button_post(keypad_name, button_name, level, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.activate_button_post_with_http_info(keypad_name, button_name, level, **kwargs)  # noqa: E501

    def activate_button_post_with_http_info(self, keypad_name, button_name, level, **kwargs):  # noqa: E501
        """Activate a button  # noqa: E501

        Activates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.activate_button_post_with_http_info(keypad_name, button_name, level, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_name',
            'button_name',
            'level',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method activate_button_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_name' is set
        if self.api_client.client_side_validation and ('keypad_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_name` when calling `activate_button_post`")  # noqa: E501
        # verify the required parameter 'button_name' is set
        if self.api_client.client_side_validation and ('button_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_name` when calling `activate_button_post`")  # noqa: E501
        # verify the required parameter 'level' is set
        if self.api_client.client_side_validation and ('level' not in local_var_params or  # noqa: E501
                                                        local_var_params['level'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `level` when calling `activate_button_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'keypad_name' in local_var_params and local_var_params['keypad_name'] is not None:  # noqa: E501
            query_params.append(('KeypadName', local_var_params['keypad_name']))  # noqa: E501
        if 'button_name' in local_var_params and local_var_params['button_name'] is not None:  # noqa: E501
            query_params.append(('ButtonName', local_var_params['button_name']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'level' in local_var_params:
            body_params = local_var_params['level']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/ActivateButton', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def deactivate_button_post(self, keypad_name, button_name, level, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Deactivates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.deactivate_button_post(keypad_name, button_name, level, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.deactivate_button_post_with_http_info(keypad_name, button_name, level, **kwargs)  # noqa: E501

    def deactivate_button_post_with_http_info(self, keypad_name, button_name, level, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Deactivates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.deactivate_button_post_with_http_info(keypad_name, button_name, level, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_name',
            'button_name',
            'level',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method deactivate_button_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_name' is set
        if self.api_client.client_side_validation and ('keypad_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_name` when calling `deactivate_button_post`")  # noqa: E501
        # verify the required parameter 'button_name' is set
        if self.api_client.client_side_validation and ('button_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_name` when calling `deactivate_button_post`")  # noqa: E501
        # verify the required parameter 'level' is set
        if self.api_client.client_side_validation and ('level' not in local_var_params or  # noqa: E501
                                                        local_var_params['level'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `level` when calling `deactivate_button_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'keypad_name' in local_var_params and local_var_params['keypad_name'] is not None:  # noqa: E501
            query_params.append(('KeypadName', local_var_params['keypad_name']))  # noqa: E501
        if 'button_name' in local_var_params and local_var_params['button_name'] is not None:  # noqa: E501
            query_params.append(('ButtonName', local_var_params['button_name']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'level' in local_var_params:
            body_params = local_var_params['level']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/DeactivateButton', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_get(self, **kwargs):  # noqa: E501
        """Get keypads  # noqa: E501

        Gets the list of Ketra keypads.  By default, unless the includeall parameter is provided and set to true, cascaded and mirrored keypads are not returned in the list. However, the buttons of cascaded keypads are returned with their respective master keypad.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_get(async_req=True)
        >>> result = thread.get()

        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param name: If specified, returns only the keypads matching the name provided
        :type name: str
        :param includeall: if true, cascaded and mirrored keypads will be returned in the list.  If false or the parameter is not provided, cascaded keypad buttons will be returned as children of their master keypad.
        :type includeall: bool
        :param nobuttons: if true, the buttons array will not be populated and only the keypad objects will be returned.  For an installation with a large number of keypads this can be much quicker than returning the full set of keypads along with all buttons.
        :type nobuttons: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2008
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_get_with_http_info(**kwargs)  # noqa: E501

    def keypads_get_with_http_info(self, **kwargs):  # noqa: E501
        """Get keypads  # noqa: E501

        Gets the list of Ketra keypads.  By default, unless the includeall parameter is provided and set to true, cascaded and mirrored keypads are not returned in the list. However, the buttons of cascaded keypads are returned with their respective master keypad.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param name: If specified, returns only the keypads matching the name provided
        :type name: str
        :param includeall: if true, cascaded and mirrored keypads will be returned in the list.  If false or the parameter is not provided, cascaded keypad buttons will be returned as children of their master keypad.
        :type includeall: bool
        :param nobuttons: if true, the buttons array will not be populated and only the keypad objects will be returned.  For an installation with a large number of keypads this can be much quicker than returning the full set of keypads along with all buttons.
        :type nobuttons: bool
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2008, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'basicauthuser',
            'basicauthpassword',
            'name',
            'includeall',
            'nobuttons'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_get" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'name' in local_var_params and local_var_params['name'] is not None:  # noqa: E501
            query_params.append(('name', local_var_params['name']))  # noqa: E501
        if 'includeall' in local_var_params and local_var_params['includeall'] is not None:  # noqa: E501
            query_params.append(('includeall', local_var_params['includeall']))  # noqa: E501
        if 'nobuttons' in local_var_params and local_var_params['nobuttons'] is not None:  # noqa: E501
            query_params.append(('nobuttons', local_var_params['nobuttons']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2008',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_buttons_button_id_activate_post(self, keypad_id, button_id, level, **kwargs):  # noqa: E501
        """Activate a button  # noqa: E501

        Activates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_activate_post(keypad_id, button_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_buttons_button_id_activate_post_with_http_info(keypad_id, button_id, level, **kwargs)  # noqa: E501

    def keypads_keypad_id_buttons_button_id_activate_post_with_http_info(self, keypad_id, button_id, level, **kwargs):  # noqa: E501
        """Activate a button  # noqa: E501

        Activates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_activate_post_with_http_info(keypad_id, button_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'button_id',
            'level',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_buttons_button_id_activate_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_buttons_button_id_activate_post`")  # noqa: E501
        # verify the required parameter 'button_id' is set
        if self.api_client.client_side_validation and ('button_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_id` when calling `keypads_keypad_id_buttons_button_id_activate_post`")  # noqa: E501
        # verify the required parameter 'level' is set
        if self.api_client.client_side_validation and ('level' not in local_var_params or  # noqa: E501
                                                        local_var_params['level'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `level` when calling `keypads_keypad_id_buttons_button_id_activate_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501
        if 'button_id' in local_var_params:
            path_params['button-id'] = local_var_params['button_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'level' in local_var_params:
            body_params = local_var_params['level']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/Buttons/{button-id}/Activate', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_buttons_button_id_deactivate_post(self, keypad_id, button_id, level, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Deactivates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_deactivate_post(keypad_id, button_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_buttons_button_id_deactivate_post_with_http_info(keypad_id, button_id, level, **kwargs)  # noqa: E501

    def keypads_keypad_id_buttons_button_id_deactivate_post_with_http_info(self, keypad_id, button_id, level, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Deactivates the scene or show represented by the button   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_deactivate_post_with_http_info(keypad_id, button_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param level: The level (required)
        :type level: Level
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'button_id',
            'level',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_buttons_button_id_deactivate_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_buttons_button_id_deactivate_post`")  # noqa: E501
        # verify the required parameter 'button_id' is set
        if self.api_client.client_side_validation and ('button_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_id` when calling `keypads_keypad_id_buttons_button_id_deactivate_post`")  # noqa: E501
        # verify the required parameter 'level' is set
        if self.api_client.client_side_validation and ('level' not in local_var_params or  # noqa: E501
                                                        local_var_params['level'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `level` when calling `keypads_keypad_id_buttons_button_id_deactivate_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501
        if 'button_id' in local_var_params:
            path_params['button-id'] = local_var_params['button_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'level' in local_var_params:
            body_params = local_var_params['level']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/Buttons/{button-id}/Deactivate', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_buttons_button_id_get(self, keypad_id, button_id, **kwargs):  # noqa: E501
        """Gets the specified button for a specified keypad  # noqa: E501

        Gets the button specified by {button-id} for the keypad specified by {keypad-id}.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_get(keypad_id, button_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse20011
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_buttons_button_id_get_with_http_info(keypad_id, button_id, **kwargs)  # noqa: E501

    def keypads_keypad_id_buttons_button_id_get_with_http_info(self, keypad_id, button_id, **kwargs):  # noqa: E501
        """Gets the specified button for a specified keypad  # noqa: E501

        Gets the button specified by {button-id} for the keypad specified by {keypad-id}.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_get_with_http_info(keypad_id, button_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse20011, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'button_id',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_buttons_button_id_get" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_buttons_button_id_get`")  # noqa: E501
        # verify the required parameter 'button_id' is set
        if self.api_client.client_side_validation and ('button_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_id` when calling `keypads_keypad_id_buttons_button_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501
        if 'button_id' in local_var_params:
            path_params['button-id'] = local_var_params['button_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/Buttons/{button-id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse20011',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_buttons_button_id_push_button_post(self, keypad_id, button_id, **kwargs):  # noqa: E501
        """Pushes a button  # noqa: E501

        Performs the same action as a physical press of the button.  The action performed is determined by the current state of the button and the type of keypad.   Added in hub firmware version 1.12 (API schema 2).   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_push_button_post(keypad_id, button_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param idempotency_key: optional string to guarantee this action will execute only once on the server.  Set this to a random string  and future requests with the same string will be ignored
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_buttons_button_id_push_button_post_with_http_info(keypad_id, button_id, **kwargs)  # noqa: E501

    def keypads_keypad_id_buttons_button_id_push_button_post_with_http_info(self, keypad_id, button_id, **kwargs):  # noqa: E501
        """Pushes a button  # noqa: E501

        Performs the same action as a physical press of the button.  The action performed is determined by the current state of the button and the type of keypad.   Added in hub firmware version 1.12 (API schema 2).   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_button_id_push_button_post_with_http_info(keypad_id, button_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param button_id: The button's name or unique identifier (uuid) (required)
        :type button_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param idempotency_key: optional string to guarantee this action will execute only once on the server.  Set this to a random string  and future requests with the same string will be ignored
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'button_id',
            'basicauthuser',
            'basicauthpassword',
            'idempotency_key'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_buttons_button_id_push_button_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_buttons_button_id_push_button_post`")  # noqa: E501
        # verify the required parameter 'button_id' is set
        if self.api_client.client_side_validation and ('button_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_id` when calling `keypads_keypad_id_buttons_button_id_push_button_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501
        if 'button_id' in local_var_params:
            path_params['button-id'] = local_var_params['button_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'idempotency_key' in local_var_params and local_var_params['idempotency_key'] is not None:  # noqa: E501
            query_params.append(('idempotency_key', local_var_params['idempotency_key']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/Buttons/{button-id}/PushButton', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_buttons_get(self, keypad_id, **kwargs):  # noqa: E501
        """Gets the buttons for a keypad  # noqa: E501

        Gets the buttons for the keypad specified by {keypad-id}.  If a keypad name is specified instead of a uuid, the buttons for the first keypad matching the specified name will be returned   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_get(keypad_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param name: If specified, returns only the buttons matching the name provided
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse20010
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_buttons_get_with_http_info(keypad_id, **kwargs)  # noqa: E501

    def keypads_keypad_id_buttons_get_with_http_info(self, keypad_id, **kwargs):  # noqa: E501
        """Gets the buttons for a keypad  # noqa: E501

        Gets the buttons for the keypad specified by {keypad-id}.  If a keypad name is specified instead of a uuid, the buttons for the first keypad matching the specified name will be returned   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_buttons_get_with_http_info(keypad_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param name: If specified, returns only the buttons matching the name provided
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse20010, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'basicauthuser',
            'basicauthpassword',
            'name'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_buttons_get" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_buttons_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'name' in local_var_params and local_var_params['name'] is not None:  # noqa: E501
            query_params.append(('name', local_var_params['name']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/Buttons', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse20010',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_get(self, keypad_id, **kwargs):  # noqa: E501
        """Gets a single keypad  # noqa: E501

        Gets a Ketra keypad speficied by {keypad-id}.  If a keypad name is specified instead of a uuid, the first keypad matching the specified name will be returned   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_get(keypad_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_get_with_http_info(keypad_id, **kwargs)  # noqa: E501

    def keypads_keypad_id_get_with_http_info(self, keypad_id, **kwargs):  # noqa: E501
        """Gets a single keypad  # noqa: E501

        Gets a Ketra keypad speficied by {keypad-id}.  If a keypad name is specified instead of a uuid, the first keypad matching the specified name will be returned   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_get_with_http_info(keypad_id, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_get" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def keypads_keypad_id_set_level_post(self, keypad_id, level, **kwargs):  # noqa: E501
        """sets the master intensity level for a single keypad  # noqa: E501

        Sets the keypad's intensity (brightness) slider to the level specified by the level parameter  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_set_level_post(keypad_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param level: The level (required)
        :type level: bool
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.keypads_keypad_id_set_level_post_with_http_info(keypad_id, level, **kwargs)  # noqa: E501

    def keypads_keypad_id_set_level_post_with_http_info(self, keypad_id, level, **kwargs):  # noqa: E501
        """sets the master intensity level for a single keypad  # noqa: E501

        Sets the keypad's intensity (brightness) slider to the level specified by the level parameter  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.keypads_keypad_id_set_level_post_with_http_info(keypad_id, level, async_req=True)
        >>> result = thread.get()

        :param keypad_id: The keypad's name or unique identifier (uuid) (required)
        :type keypad_id: str
        :param level: The level (required)
        :type level: bool
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_id',
            'level',
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method keypads_keypad_id_set_level_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_id' is set
        if self.api_client.client_side_validation and ('keypad_id' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_id'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_id` when calling `keypads_keypad_id_set_level_post`")  # noqa: E501
        # verify the required parameter 'level' is set
        if self.api_client.client_side_validation and ('level' not in local_var_params or  # noqa: E501
                                                        local_var_params['level'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `level` when calling `keypads_keypad_id_set_level_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'keypad_id' in local_var_params:
            path_params['keypad-id'] = local_var_params['keypad_id']  # noqa: E501

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'level' in local_var_params and local_var_params['level'] is not None:  # noqa: E501
            query_params.append(('Level', local_var_params['level']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/Keypads/{keypad-id}/SetLevel', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def push_button_post(self, keypad_name, button_name, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Performs the same action as a physical press of the button.  The action performed is determined by the current state of the button and the type of keypad.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.push_button_post(keypad_name, button_name, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param idempotency_key: optional string to guarantee this action will execute only once on the server.  Set this to a random string  and future requests with the same string will be ignored
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse2009
        """
        kwargs['_return_http_data_only'] = True
        return self.push_button_post_with_http_info(keypad_name, button_name, **kwargs)  # noqa: E501

    def push_button_post_with_http_info(self, keypad_name, button_name, **kwargs):  # noqa: E501
        """Deactivate a button  # noqa: E501

        Performs the same action as a physical press of the button.  The action performed is determined by the current state of the button and the type of keypad.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.push_button_post_with_http_info(keypad_name, button_name, async_req=True)
        >>> result = thread.get()

        :param keypad_name: Specifies a keypad name (required)
        :type keypad_name: str
        :param button_name: Specifies a button name (required)
        :type button_name: str
        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param idempotency_key: optional string to guarantee this action will execute only once on the server.  Set this to a random string  and future requests with the same string will be ignored
        :type idempotency_key: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse2009, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'keypad_name',
            'button_name',
            'basicauthuser',
            'basicauthpassword',
            'idempotency_key'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method push_button_post" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'keypad_name' is set
        if self.api_client.client_side_validation and ('keypad_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['keypad_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `keypad_name` when calling `push_button_post`")  # noqa: E501
        # verify the required parameter 'button_name' is set
        if self.api_client.client_side_validation and ('button_name' not in local_var_params or  # noqa: E501
                                                        local_var_params['button_name'] is None):  # noqa: E501
            raise ApiValueError("Missing the required parameter `button_name` when calling `push_button_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501
        if 'keypad_name' in local_var_params and local_var_params['keypad_name'] is not None:  # noqa: E501
            query_params.append(('KeypadName', local_var_params['keypad_name']))  # noqa: E501
        if 'button_name' in local_var_params and local_var_params['button_name'] is not None:  # noqa: E501
            query_params.append(('ButtonName', local_var_params['button_name']))  # noqa: E501
        if 'idempotency_key' in local_var_params and local_var_params['idempotency_key'] is not None:  # noqa: E501
            query_params.append(('idempotency_key', local_var_params['idempotency_key']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/PushButton', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse2009',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))

    def root_get(self, **kwargs):  # noqa: E501
        """Get keypads and groups  (and scenes in API schema 4 or later)  # noqa: E501

        Gets all keypads and groups in the installation.  Added in hub firmware version 1.14 (API schema 3).   Scenes are also returned in API schema 4.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.root_get(async_req=True)
        >>> result = thread.get()

        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: InlineResponse200
        """
        kwargs['_return_http_data_only'] = True
        return self.root_get_with_http_info(**kwargs)  # noqa: E501

    def root_get_with_http_info(self, **kwargs):  # noqa: E501
        """Get keypads and groups  (and scenes in API schema 4 or later)  # noqa: E501

        Gets all keypads and groups in the installation.  Added in hub firmware version 1.14 (API schema 3).   Scenes are also returned in API schema 4.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.root_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param basicauthuser: Username to use in place of username in basic authentication header.  For a secure installation, this value is ignored but still must be supplied unless a basic authentication header is sent with the request.
        :type basicauthuser: str
        :param basicauthpassword: Password to use in place of password in basic authentication header. For a secure installation, this should be an oauth token for a user with access to the installation.  If a basic authentication header is sent, this parameter is ignored.  If no basic authentication header is sent, this parameter as well as the basicauthuser parameter must be supplied if the hub is a member of a secure installation.
        :type basicauthpassword: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(InlineResponse200, status_code(int), headers(HTTPHeaderDict))
        """

        local_var_params = locals()

        all_params = [
            'basicauthuser',
            'basicauthpassword'
        ]
        all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth'
            ]
        )

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method root_get" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'basicauthuser' in local_var_params and local_var_params['basicauthuser'] is not None:  # noqa: E501
            query_params.append(('basicauthuser', local_var_params['basicauthuser']))  # noqa: E501
        if 'basicauthpassword' in local_var_params and local_var_params['basicauthpassword'] is not None:  # noqa: E501
            query_params.append(('basicauthpassword', local_var_params['basicauthpassword']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['basicAuth']  # noqa: E501

        return self.api_client.call_api(
            '/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='InlineResponse200',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats,
            _request_auth=local_var_params.get('_request_auth'))



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import aioketraapi
from aioketraapi.models.inline_response200_content import InlineResponse200Content  # noqa: E501
from aioketraapi.rest import ApiException

class TestInlineResponse200Content(unittest.TestCase):
    """InlineResponse200Content unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test InlineResponse200Content
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = aioketraapi.models.inline_response200_content.InlineResponse200Content()  # noqa: E501
        if include_optional :
            return InlineResponse200Content(
                keypads = [
                    aioketraapi.models.keypad.Keypad(
                        id = '0', 
                        is_virtual = True, 
                        level = 56, 
                        name = '0', 
                        serial_number = '0', 
                        buttons = [
                            aioketraapi.models.button.Button(
                                active = True, 
                                id = '0', 
                                level = 56, 
                                name = '0', 
                                position = 56, 
                                is_power_button = True, 
                                scene_is_modified = True, )
                            ], )
                    ], 
                groups = [
                    aioketraapi.models.group.Group(
                        id = '0', 
                        name = '0', 
                        child_groups = [
                            '0'
                            ], 
                        parent_groups = [
                            '0'
                            ], 
                        address = 56, 
                        state = aioketraapi.models.lamp_state.LampState(), )
                    ], 
                scenes = [
                    aioketraapi.models.scene.Scene(
                        id = '0', 
                        content_id = 56, 
                        is_show = True, 
                        show_group_number = 56, 
                        name = '0', 
                        parent_group_ids = [
                            '0'
                            ], )
                    ]
            )
        else :
            return InlineResponse200Content(
        )

    def testInlineResponse200Content(self):
        """Test InlineResponse200Content"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from aioketraapi.configuration import Configuration


class Scene(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'content_id': 'int',
        'is_show': 'bool',
        'show_group_number': 'int',
        'name': 'str',
        'parent_group_ids': 'list[str]'
    }

    attribute_map = {
        'id': 'Id',
        'content_id': 'ContentId',
        'is_show': 'IsShow',
        'show_group_number': 'ShowGroupNumber',
        'name': 'Name',
        'parent_group_ids': 'ParentGroupIds'
    }

    def __init__(self, id=None, content_id=None, is_show=None, show_group_number=None, name=None, parent_group_ids=None, local_vars_configuration=None):  # noqa: E501
        """Scene - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._content_id = None
        self._is_show = None
        self._show_group_number = None
        self._name = None
        self._parent_group_ids = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if content_id is not None:
            self.content_id = content_id
        if is_show is not None:
            self.is_show = is_show
        if show_group_number is not None:
            self.show_group_number = show_group_number
        if name is not None:
            self.name = name
        if parent_group_ids is not None:
            self.parent_group_ids = parent_group_ids

    @property
    def id(self):
        """Gets the id of this Scene.  # noqa: E501

        Unique id of the scene  # noqa: E501

        :return: The id of this Scene.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Scene.

        Unique id of the scene  # noqa: E501

        :param id: The id of this Scene.  # noqa: E501
        :type id: str
        """

        self._id = id

    @property
    def content_id(self):
        """Gets the content_id of this Scene.  # noqa: E501

        content id for the scene  # noqa: E501

        :return: The content_id of this Scene.  # noqa: E501
        :rtype: int
        """
        return self._content_id

    @content_id.setter
    def content_id(self, content_id):
        """Sets the content_id of this Scene.

        content id for the scene  # noqa: E501

        :param content_id: The content_id of this Scene.  # noqa: E501
        :type content_id: int
        """

        self._content_id = content_id

    @property
    def is_show(self):
        """Gets the is_show of this Scene.  # noqa: E501

        true if the scene represents a trigger for a show (or group of shows)  # noqa: E501

        :return: The is_show of this Scene.  # noqa: E501
        :rtype: bool
        """
        return self._is_show

    @is_show.setter
    def is_show(self, is_show):
        """Sets the is_show of this Scene.

        true if the scene represents a trigger for a show (or group of shows)  # noqa: E501

        :param is_show: The is_show of this Scene.  # noqa: E501
        :type is_show: bool
        """

        self._is_show = is_show

    @property
    def show_group_number(self):
        """Gets the show_group_number of this Scene.  # noqa: E501

        if the scene represents a show trigger, this value indicates which show group is activated by this scene.  # noqa: E501

        :return: The show_group_number of this Scene.  # noqa: E501
        :rtype: int
        """
        return self._show_group_number

    @show_group_number.setter
    def show_group_number(self, show_group_number):
        """Sets the show_group_number of this Scene.

        if the scene represents a show trigger, this value indicates which show group is activated by this scene.  # noqa: E501

        :param show_group_number: The show_group_number of this Scene.  # noqa: E501
        :type show_group_number: int
        """

        self._show_group_number = show_group_number

    @property
    def name(self):
        """Gets the name of this Scene.  # noqa: E501

        Name of the scene  # noqa: E501

        :return: The name of this Scene.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Scene.

        Name of the scene  # noqa: E501

        :param name: The name of this Scene.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def parent_group_ids(self):
        """Gets the parent_group_ids of this Scene.  # noqa: E501

        Ids of parent groups of this scene.  This indicates which groups are valid for which the scene can be active.  # noqa: E501

        :return: The parent_group_ids of this Scene.  # noqa: E501
        :rtype: list[str]
        """
        return self._parent_group_ids

    @parent_group_ids.setter
    def parent_group_ids(self, parent_group_ids):
        """Sets the parent_group_ids of this Scene.

        Ids of parent groups of this scene.  This indicates which groups are valid for which the scene can be active.  # noqa: E501

        :param parent_group_ids: The parent_group_ids of this Scene.  # noqa: E501
        :type parent_group_ids: list[str]
        """

        self._parent_group_ids = parent_group_ids

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Scene):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Scene):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from aioketraapi.configuration import Configuration


class LampState(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'brightness': 'float',
        'master_brightness': 'float',
        'net_brightness': 'float',
        'dim_curve_active': 'bool',
        'dim_curve': 'str',
        'power_on': 'bool',
        'vibrancy': 'float',
        'cct': 'int',
        'x_chromaticity': 'float',
        'y_chromaticity': 'float',
        'transition_time': 'int',
        'updated_at': 'str',
        'transition_complete': 'bool',
        'active_shows': 'list[int]',
        'start_state': 'LampStateParameters'
    }

    attribute_map = {
        'brightness': 'Brightness',
        'master_brightness': 'MasterBrightness',
        'net_brightness': 'NetBrightness',
        'dim_curve_active': 'DimCurveActive',
        'dim_curve': 'DimCurve',
        'power_on': 'PowerOn',
        'vibrancy': 'Vibrancy',
        'cct': 'CCT',
        'x_chromaticity': 'xChromaticity',
        'y_chromaticity': 'yChromaticity',
        'transition_time': 'TransitionTime',
        'updated_at': 'UpdatedAt',
        'transition_complete': 'TransitionComplete',
        'active_shows': 'ActiveShows',
        'start_state': 'StartState'
    }

    def __init__(self, brightness=None, master_brightness=None, net_brightness=None, dim_curve_active=None, dim_curve=None, power_on=None, vibrancy=None, cct=None, x_chromaticity=None, y_chromaticity=None, transition_time=None, updated_at=None, transition_complete=None, active_shows=None, start_state=None, local_vars_configuration=None):  # noqa: E501
        """LampState - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._brightness = None
        self._master_brightness = None
        self._net_brightness = None
        self._dim_curve_active = None
        self._dim_curve = None
        self._power_on = None
        self._vibrancy = None
        self._cct = None
        self._x_chromaticity = None
        self._y_chromaticity = None
        self._transition_time = None
        self._updated_at = None
        self._transition_complete = None
        self._active_shows = None
        self._start_state = None
        self.discriminator = None

        if brightness is not None:
            self.brightness = brightness
        if master_brightness is not None:
            self.master_brightness = master_brightness
        if net_brightness is not None:
            self.net_brightness = net_brightness
        if dim_curve_active is not None:
            self.dim_curve_active = dim_curve_active
        if dim_curve is not None:
            self.dim_curve = dim_curve
        if power_on is not None:
            self.power_on = power_on
        if vibrancy is not None:
            self.vibrancy = vibrancy
        if cct is not None:
            self.cct = cct
        if x_chromaticity is not None:
            self.x_chromaticity = x_chromaticity
        if y_chromaticity is not None:
            self.y_chromaticity = y_chromaticity
        if transition_time is not None:
            self.transition_time = transition_time
        if updated_at is not None:
            self.updated_at = updated_at
        if transition_complete is not None:
            self.transition_complete = transition_complete
        if active_shows is not None:
            self.active_shows = active_shows
        if start_state is not None:
            self.start_state = start_state

    @property
    def brightness(self):
        """Gets the brightness of this LampState.  # noqa: E501

        the brightness of the light  # noqa: E501

        :return: The brightness of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._brightness

    @brightness.setter
    def brightness(self, brightness):
        """Sets the brightness of this LampState.

        the brightness of the light  # noqa: E501

        :param brightness: The brightness of this LampState.  # noqa: E501
        :type brightness: float
        """
        if (self.local_vars_configuration.client_side_validation and
                brightness is not None and brightness > 1):  # noqa: E501
            raise ValueError("Invalid value for `brightness`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                brightness is not None and brightness < 0):  # noqa: E501
            raise ValueError("Invalid value for `brightness`, must be a value greater than or equal to `0`")  # noqa: E501

        self._brightness = brightness

    @property
    def master_brightness(self):
        """Gets the master_brightness of this LampState.  # noqa: E501

        the master brightness value of the light.  This parameter is new for API schema 4 and provides an independent brightness channel that can be varied while 'Brightness' remains constant.  # noqa: E501

        :return: The master_brightness of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._master_brightness

    @master_brightness.setter
    def master_brightness(self, master_brightness):
        """Sets the master_brightness of this LampState.

        the master brightness value of the light.  This parameter is new for API schema 4 and provides an independent brightness channel that can be varied while 'Brightness' remains constant.  # noqa: E501

        :param master_brightness: The master_brightness of this LampState.  # noqa: E501
        :type master_brightness: float
        """
        if (self.local_vars_configuration.client_side_validation and
                master_brightness is not None and master_brightness > 1):  # noqa: E501
            raise ValueError("Invalid value for `master_brightness`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                master_brightness is not None and master_brightness < 0):  # noqa: E501
            raise ValueError("Invalid value for `master_brightness`, must be a value greater than or equal to `0`")  # noqa: E501

        self._master_brightness = master_brightness

    @property
    def net_brightness(self):
        """Gets the net_brightness of this LampState.  # noqa: E501

        The overall brightness of the light (Brightness * MasterBrightness). This parameter is new for API schema 4 and is ignored for a PUT or POST operation.  # noqa: E501

        :return: The net_brightness of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._net_brightness

    @net_brightness.setter
    def net_brightness(self, net_brightness):
        """Sets the net_brightness of this LampState.

        The overall brightness of the light (Brightness * MasterBrightness). This parameter is new for API schema 4 and is ignored for a PUT or POST operation.  # noqa: E501

        :param net_brightness: The net_brightness of this LampState.  # noqa: E501
        :type net_brightness: float
        """
        if (self.local_vars_configuration.client_side_validation and
                net_brightness is not None and net_brightness > 1):  # noqa: E501
            raise ValueError("Invalid value for `net_brightness`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                net_brightness is not None and net_brightness < 0):  # noqa: E501
            raise ValueError("Invalid value for `net_brightness`, must be a value greater than or equal to `0`")  # noqa: E501

        self._net_brightness = net_brightness

    @property
    def dim_curve_active(self):
        """Gets the dim_curve_active of this LampState.  # noqa: E501

        returns true if a dimming curve is currently active on the lamp group.  New for API schema 4.  # noqa: E501

        :return: The dim_curve_active of this LampState.  # noqa: E501
        :rtype: bool
        """
        return self._dim_curve_active

    @dim_curve_active.setter
    def dim_curve_active(self, dim_curve_active):
        """Sets the dim_curve_active of this LampState.

        returns true if a dimming curve is currently active on the lamp group.  New for API schema 4.  # noqa: E501

        :param dim_curve_active: The dim_curve_active of this LampState.  # noqa: E501
        :type dim_curve_active: bool
        """

        self._dim_curve_active = dim_curve_active

    @property
    def dim_curve(self):
        """Gets the dim_curve of this LampState.  # noqa: E501

        if DimCurveActive is true, indicates which dimming curve is active.  New for API schema 4.  # noqa: E501

        :return: The dim_curve of this LampState.  # noqa: E501
        :rtype: str
        """
        return self._dim_curve

    @dim_curve.setter
    def dim_curve(self, dim_curve):
        """Sets the dim_curve of this LampState.

        if DimCurveActive is true, indicates which dimming curve is active.  New for API schema 4.  # noqa: E501

        :param dim_curve: The dim_curve of this LampState.  # noqa: E501
        :type dim_curve: str
        """
        allowed_values = ["Xenon", "GE Ultra Soft White", "Philips EcoVantage A19", "Philips Halogen PAR38", "none"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and dim_curve not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `dim_curve` ({0}), must be one of {1}"  # noqa: E501
                .format(dim_curve, allowed_values)
            )

        self._dim_curve = dim_curve

    @property
    def power_on(self):
        """Gets the power_on of this LampState.  # noqa: E501

        true if the light is on, false if the light is off  # noqa: E501

        :return: The power_on of this LampState.  # noqa: E501
        :rtype: bool
        """
        return self._power_on

    @power_on.setter
    def power_on(self, power_on):
        """Sets the power_on of this LampState.

        true if the light is on, false if the light is off  # noqa: E501

        :param power_on: The power_on of this LampState.  # noqa: E501
        :type power_on: bool
        """

        self._power_on = power_on

    @property
    def vibrancy(self):
        """Gets the vibrancy of this LampState.  # noqa: E501

        the ratio of RGB to phosphor-converted white LED content.  A value of 0 indicates to use as little RGB as possible.  A value of 1 indicates to use as much RGB as possible  # noqa: E501

        :return: The vibrancy of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._vibrancy

    @vibrancy.setter
    def vibrancy(self, vibrancy):
        """Sets the vibrancy of this LampState.

        the ratio of RGB to phosphor-converted white LED content.  A value of 0 indicates to use as little RGB as possible.  A value of 1 indicates to use as much RGB as possible  # noqa: E501

        :param vibrancy: The vibrancy of this LampState.  # noqa: E501
        :type vibrancy: float
        """
        if (self.local_vars_configuration.client_side_validation and
                vibrancy is not None and vibrancy > 1):  # noqa: E501
            raise ValueError("Invalid value for `vibrancy`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                vibrancy is not None and vibrancy < 0):  # noqa: E501
            raise ValueError("Invalid value for `vibrancy`, must be a value greater than or equal to `0`")  # noqa: E501

        self._vibrancy = vibrancy

    @property
    def cct(self):
        """Gets the cct of this LampState.  # noqa: E501

        the correlated color temperature of the light.  If cct is provided and within the valid range, it will be used instead of x and y chromaticity  # noqa: E501

        :return: The cct of this LampState.  # noqa: E501
        :rtype: int
        """
        return self._cct

    @cct.setter
    def cct(self, cct):
        """Sets the cct of this LampState.

        the correlated color temperature of the light.  If cct is provided and within the valid range, it will be used instead of x and y chromaticity  # noqa: E501

        :param cct: The cct of this LampState.  # noqa: E501
        :type cct: int
        """
        if (self.local_vars_configuration.client_side_validation and
                cct is not None and cct > 25000):  # noqa: E501
            raise ValueError("Invalid value for `cct`, must be a value less than or equal to `25000`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                cct is not None and cct < 0):  # noqa: E501
            raise ValueError("Invalid value for `cct`, must be a value greater than or equal to `0`")  # noqa: E501

        self._cct = cct

    @property
    def x_chromaticity(self):
        """Gets the x_chromaticity of this LampState.  # noqa: E501

        the x chromaticity of the light.  This parameter will be ignored if a valid cct value is provided on a put/post operation.  A get operation will return a valid value always  # noqa: E501

        :return: The x_chromaticity of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._x_chromaticity

    @x_chromaticity.setter
    def x_chromaticity(self, x_chromaticity):
        """Sets the x_chromaticity of this LampState.

        the x chromaticity of the light.  This parameter will be ignored if a valid cct value is provided on a put/post operation.  A get operation will return a valid value always  # noqa: E501

        :param x_chromaticity: The x_chromaticity of this LampState.  # noqa: E501
        :type x_chromaticity: float
        """
        if (self.local_vars_configuration.client_side_validation and
                x_chromaticity is not None and x_chromaticity > 1):  # noqa: E501
            raise ValueError("Invalid value for `x_chromaticity`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                x_chromaticity is not None and x_chromaticity < 0):  # noqa: E501
            raise ValueError("Invalid value for `x_chromaticity`, must be a value greater than or equal to `0`")  # noqa: E501

        self._x_chromaticity = x_chromaticity

    @property
    def y_chromaticity(self):
        """Gets the y_chromaticity of this LampState.  # noqa: E501

        the y chromaticity of the light.  This parameter will be ignored if a valid cct value is provided on a put/post operation.  A get operation will return a valid value always  # noqa: E501

        :return: The y_chromaticity of this LampState.  # noqa: E501
        :rtype: float
        """
        return self._y_chromaticity

    @y_chromaticity.setter
    def y_chromaticity(self, y_chromaticity):
        """Sets the y_chromaticity of this LampState.

        the y chromaticity of the light.  This parameter will be ignored if a valid cct value is provided on a put/post operation.  A get operation will return a valid value always  # noqa: E501

        :param y_chromaticity: The y_chromaticity of this LampState.  # noqa: E501
        :type y_chromaticity: float
        """
        if (self.local_vars_configuration.client_side_validation and
                y_chromaticity is not None and y_chromaticity > 1):  # noqa: E501
            raise ValueError("Invalid value for `y_chromaticity`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                y_chromaticity is not None and y_chromaticity < 0):  # noqa: E501
            raise ValueError("Invalid value for `y_chromaticity`, must be a value greater than or equal to `0`")  # noqa: E501

        self._y_chromaticity = y_chromaticity

    @property
    def transition_time(self):
        """Gets the transition_time of this LampState.  # noqa: E501

        transition time, in ms.  (Required)  # noqa: E501

        :return: The transition_time of this LampState.  # noqa: E501
        :rtype: int
        """
        return self._transition_time

    @transition_time.setter
    def transition_time(self, transition_time):
        """Sets the transition_time of this LampState.

        transition time, in ms.  (Required)  # noqa: E501

        :param transition_time: The transition_time of this LampState.  # noqa: E501
        :type transition_time: int
        """
        if (self.local_vars_configuration.client_side_validation and
                transition_time is not None and transition_time > 6553500):  # noqa: E501
            raise ValueError("Invalid value for `transition_time`, must be a value less than or equal to `6553500`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                transition_time is not None and transition_time < 0):  # noqa: E501
            raise ValueError("Invalid value for `transition_time`, must be a value greater than or equal to `0`")  # noqa: E501

        self._transition_time = transition_time

    @property
    def updated_at(self):
        """Gets the updated_at of this LampState.  # noqa: E501

        the time at which the group was last updated.  New for API schema 4.  # noqa: E501

        :return: The updated_at of this LampState.  # noqa: E501
        :rtype: str
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this LampState.

        the time at which the group was last updated.  New for API schema 4.  # noqa: E501

        :param updated_at: The updated_at of this LampState.  # noqa: E501
        :type updated_at: str
        """

        self._updated_at = updated_at

    @property
    def transition_complete(self):
        """Gets the transition_complete of this LampState.  # noqa: E501

        returns true if the lamp is finished transitioning to the last updated color.  New for API schema 4.  # noqa: E501

        :return: The transition_complete of this LampState.  # noqa: E501
        :rtype: bool
        """
        return self._transition_complete

    @transition_complete.setter
    def transition_complete(self, transition_complete):
        """Sets the transition_complete of this LampState.

        returns true if the lamp is finished transitioning to the last updated color.  New for API schema 4.  # noqa: E501

        :param transition_complete: The transition_complete of this LampState.  # noqa: E501
        :type transition_complete: bool
        """

        self._transition_complete = transition_complete

    @property
    def active_shows(self):
        """Gets the active_shows of this LampState.  # noqa: E501

        indicates the show groups that are currently active for the group.  New for API schema 4.  # noqa: E501

        :return: The active_shows of this LampState.  # noqa: E501
        :rtype: list[int]
        """
        return self._active_shows

    @active_shows.setter
    def active_shows(self, active_shows):
        """Sets the active_shows of this LampState.

        indicates the show groups that are currently active for the group.  New for API schema 4.  # noqa: E501

        :param active_shows: The active_shows of this LampState.  # noqa: E501
        :type active_shows: list[int]
        """

        self._active_shows = active_shows

    @property
    def start_state(self):
        """Gets the start_state of this LampState.  # noqa: E501


        :return: The start_state of this LampState.  # noqa: E501
        :rtype: LampStateParameters
        """
        return self._start_state

    @start_state.setter
    def start_state(self, start_state):
        """Sets the start_state of this LampState.


        :param start_state: The start_state of this LampState.  # noqa: E501
        :type start_state: LampStateParameters
        """

        self._start_state = start_state

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LampState):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LampState):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""

import sys
from setuptools import setup, find_packages  # noqa: H301
import pathlib

if sys.version_info < (3, 6):
    raise RuntimeError("aioketraapi requires Python 3.6+")

HERE = pathlib.Path(__file__).parent

NAME = "aioketraapi"
VERSION = "0.1.12"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil"]
REQUIRES.append("aiohttp >= 3.0.0")


def read(f):
    return (HERE / f).read_text("utf-8").strip()


setup(
    name=NAME,
    version=VERSION,
    description="Ketra Lighting API",
    author="Joe Savage",
    author_email="joe@savage.zone",
    url="https://github.com/s4v4g3/aio-ketra-api.git",
    keywords=["OpenAPI", "OpenAPI-Generator", "Ketra Lighting API"],
    install_requires=REQUIRES,
    packages=find_packages(exclude=["test", "tests"]),
    python_requires=">=3.6",
    include_package_data=True,
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
)



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import aioketraapi
from aioketraapi.models.inline_response2004_content import InlineResponse2004Content  # noqa: E501
from aioketraapi.rest import ApiException

class TestInlineResponse2004Content(unittest.TestCase):
    """InlineResponse2004Content unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test InlineResponse2004Content
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = aioketraapi.models.inline_response2004_content.InlineResponse2004Content()  # noqa: E501
        if include_optional :
            return InlineResponse2004Content(
                api_schema = 56, 
                cpu_version = '0', 
                radio_version = '0', 
                serial_number = '0', 
                model_name = '0', 
                id = '0', 
                installation_id = '0', 
                installation_name = '0', 
                network_id = '0', 
                name = '0', 
                ethernet_mac = '0', 
                i_pv4_address = '0', 
                ethernet_link_status = [
                    True
                    ], 
                up_time_seconds = 56, 
                local_time = '0', 
                utc_time = '0', 
                last_reboot_reason = '0', 
                has_internet_connectivity = True, 
                last_time_update_was_successful = True, 
                remote_connection_enabled = True, 
                remote_connection_established = True, 
                supports_zone_keypads = True
            )
        else :
            return InlineResponse2004Content(
        )

    def testInlineResponse2004Content(self):
        """Test InlineResponse2004Content"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import aioketraapi
from aioketraapi.models.lamp import Lamp  # noqa: E501
from aioketraapi.rest import ApiException

class TestLamp(unittest.TestCase):
    """Lamp unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test Lamp
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = aioketraapi.models.lamp.Lamp()  # noqa: E501
        if include_optional :
            return Lamp(
                id = '0', 
                name = '0', 
                channel = 56, 
                parent_group_ids = [
                    '0'
                    ], 
                serial_number = '0', 
                number_of_subchannels = 56, 
                logical_address = 56
            )
        else :
            return Lamp(
        )

    def testLamp(self):
        """Test Lamp"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

import aioketraapi
from aioketraapi.models.lamp_state import LampState  # noqa: E501
from aioketraapi.rest import ApiException

class TestLampState(unittest.TestCase):
    """LampState unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test LampState
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = aioketraapi.models.lamp_state.LampState()  # noqa: E501
        if include_optional :
            return LampState(
                brightness = 0, 
                master_brightness = 0, 
                net_brightness = 0, 
                dim_curve_active = True, 
                dim_curve = 'Xenon', 
                power_on = True, 
                vibrancy = 0, 
                cct = 0, 
                x_chromaticity = 0, 
                y_chromaticity = 0, 
                transition_time = 0, 
                updated_at = '0', 
                transition_complete = True, 
                active_shows = [
                    56
                    ], 
                start_state = aioketraapi.models.lamp_state_parameters.LampStateParameters(
                    brightness = 0, 
                    master_brightness = 0, 
                    net_brightness = 0, 
                    dim_curve_active = True, 
                    dim_curve = 'Xenon', 
                    power_on = True, 
                    vibrancy = 0, 
                    cct = 0, 
                    x_chromaticity = 0, 
                    y_chromaticity = 0, )
            )
        else :
            return LampState(
        )

    def testLampState(self):
        """Test LampState"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from aioketraapi.configuration import Configuration


class Keypad(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'is_virtual': 'bool',
        'level': 'int',
        'name': 'str',
        'serial_number': 'str',
        'buttons': 'list[Button]'
    }

    attribute_map = {
        'id': 'Id',
        'is_virtual': 'IsVirtual',
        'level': 'Level',
        'name': 'Name',
        'serial_number': 'SerialNumber',
        'buttons': 'Buttons'
    }

    def __init__(self, id=None, is_virtual=None, level=None, name=None, serial_number=None, buttons=None, local_vars_configuration=None):  # noqa: E501
        """Keypad - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._is_virtual = None
        self._level = None
        self._name = None
        self._serial_number = None
        self._buttons = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if is_virtual is not None:
            self.is_virtual = is_virtual
        if level is not None:
            self.level = level
        if name is not None:
            self.name = name
        if serial_number is not None:
            self.serial_number = serial_number
        if buttons is not None:
            self.buttons = buttons

    @property
    def id(self):
        """Gets the id of this Keypad.  # noqa: E501

        Unique id of the keypad  # noqa: E501

        :return: The id of this Keypad.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Keypad.

        Unique id of the keypad  # noqa: E501

        :param id: The id of this Keypad.  # noqa: E501
        :type id: str
        """

        self._id = id

    @property
    def is_virtual(self):
        """Gets the is_virtual of this Keypad.  # noqa: E501

        Is the keypad virtual or physical?  # noqa: E501

        :return: The is_virtual of this Keypad.  # noqa: E501
        :rtype: bool
        """
        return self._is_virtual

    @is_virtual.setter
    def is_virtual(self, is_virtual):
        """Sets the is_virtual of this Keypad.

        Is the keypad virtual or physical?  # noqa: E501

        :param is_virtual: The is_virtual of this Keypad.  # noqa: E501
        :type is_virtual: bool
        """

        self._is_virtual = is_virtual

    @property
    def level(self):
        """Gets the level of this Keypad.  # noqa: E501

        intensity level of the keypad's slider  # noqa: E501

        :return: The level of this Keypad.  # noqa: E501
        :rtype: int
        """
        return self._level

    @level.setter
    def level(self, level):
        """Sets the level of this Keypad.

        intensity level of the keypad's slider  # noqa: E501

        :param level: The level of this Keypad.  # noqa: E501
        :type level: int
        """

        self._level = level

    @property
    def name(self):
        """Gets the name of this Keypad.  # noqa: E501

        Name of the keypad  # noqa: E501

        :return: The name of this Keypad.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Keypad.

        Name of the keypad  # noqa: E501

        :param name: The name of this Keypad.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def serial_number(self):
        """Gets the serial_number of this Keypad.  # noqa: E501

        if the keypad is not virtual, specifies the serial number of the keypad  # noqa: E501

        :return: The serial_number of this Keypad.  # noqa: E501
        :rtype: str
        """
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        """Sets the serial_number of this Keypad.

        if the keypad is not virtual, specifies the serial number of the keypad  # noqa: E501

        :param serial_number: The serial_number of this Keypad.  # noqa: E501
        :type serial_number: str
        """

        self._serial_number = serial_number

    @property
    def buttons(self):
        """Gets the buttons of this Keypad.  # noqa: E501


        :return: The buttons of this Keypad.  # noqa: E501
        :rtype: list[Button]
        """
        return self._buttons

    @buttons.setter
    def buttons(self, buttons):
        """Sets the buttons of this Keypad.


        :param buttons: The buttons of this Keypad.  # noqa: E501
        :type buttons: list[Button]
        """

        self._buttons = buttons

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Keypad):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Keypad):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from aioketraapi.configuration import Configuration


class PublishBegin(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'notification_type': 'str',
        'time_utc': 'datetime'
    }

    attribute_map = {
        'notification_type': 'notification_type',
        'time_utc': 'time_utc'
    }

    def __init__(self, notification_type=None, time_utc=None, local_vars_configuration=None):  # noqa: E501
        """PublishBegin - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._notification_type = None
        self._time_utc = None
        self.discriminator = None

        self.notification_type = notification_type
        self.time_utc = time_utc

    @property
    def notification_type(self):
        """Gets the notification_type of this PublishBegin.  # noqa: E501

        type of notification  # noqa: E501

        :return: The notification_type of this PublishBegin.  # noqa: E501
        :rtype: str
        """
        return self._notification_type

    @notification_type.setter
    def notification_type(self, notification_type):
        """Sets the notification_type of this PublishBegin.

        type of notification  # noqa: E501

        :param notification_type: The notification_type of this PublishBegin.  # noqa: E501
        :type notification_type: str
        """
        if self.local_vars_configuration.client_side_validation and notification_type is None:  # noqa: E501
            raise ValueError("Invalid value for `notification_type`, must not be `None`")  # noqa: E501
        allowed_values = ["ButtonChange", "PublishBegin", "HubReady", "GroupStateChange"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and notification_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `notification_type` ({0}), must be one of {1}"  # noqa: E501
                .format(notification_type, allowed_values)
            )

        self._notification_type = notification_type

    @property
    def time_utc(self):
        """Gets the time_utc of this PublishBegin.  # noqa: E501

        time of notification  # noqa: E501

        :return: The time_utc of this PublishBegin.  # noqa: E501
        :rtype: datetime
        """
        return self._time_utc

    @time_utc.setter
    def time_utc(self, time_utc):
        """Sets the time_utc of this PublishBegin.

        time of notification  # noqa: E501

        :param time_utc: The time_utc of this PublishBegin.  # noqa: E501
        :type time_utc: datetime
        """
        if self.local_vars_configuration.client_side_validation and time_utc is None:  # noqa: E501
            raise ValueError("Invalid value for `time_utc`, must not be `None`")  # noqa: E501

        self._time_utc = time_utc

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PublishBegin):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PublishBegin):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest

import aioketraapi
from aioketraapi.api.scene_operations_api import SceneOperationsApi  # noqa: E501
from aioketraapi.rest import ApiException


class TestSceneOperationsApi(unittest.TestCase):
    """SceneOperationsApi unit test stubs"""

    def setUp(self):
        self.api = aioketraapi.api.scene_operations_api.SceneOperationsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_root_get(self):
        """Test case for root_get

        Get keypads and groups  (and scenes in API schema 4 or later)  # noqa: E501
        """
        pass

    def test_scenes_get(self):
        """Test case for scenes_get

        Get Scenes  # noqa: E501
        """
        pass

    def test_scenes_scene_id_activate_post(self):
        """Test case for scenes_scene_id_activate_post

        Activates a scene  # noqa: E501
        """
        pass

    def test_scenes_scene_id_get(self):
        """Test case for scenes_scene_id_get

        Gets a single scene  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ketra Lighting API

    Control your Ketra lights  # noqa: E501

    The version of the OpenAPI document: 1.4.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from aioketraapi.configuration import Configuration


class Lamp(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'channel': 'int',
        'parent_group_ids': 'list[str]',
        'serial_number': 'str',
        'number_of_subchannels': 'int',
        'logical_address': 'int'
    }

    attribute_map = {
        'id': 'Id',
        'name': 'Name',
        'channel': 'Channel',
        'parent_group_ids': 'ParentGroupIds',
        'serial_number': 'SerialNumber',
        'number_of_subchannels': 'NumberOfSubchannels',
        'logical_address': 'LogicalAddress'
    }

    def __init__(self, id=None, name=None, channel=None, parent_group_ids=None, serial_number=None, number_of_subchannels=None, logical_address=None, local_vars_configuration=None):  # noqa: E501
        """Lamp - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._channel = None
        self._parent_group_ids = None
        self._serial_number = None
        self._number_of_subchannels = None
        self._logical_address = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if channel is not None:
            self.channel = channel
        if parent_group_ids is not None:
            self.parent_group_ids = parent_group_ids
        if serial_number is not None:
            self.serial_number = serial_number
        if number_of_subchannels is not None:
            self.number_of_subchannels = number_of_subchannels
        if logical_address is not None:
            self.logical_address = logical_address

    @property
    def id(self):
        """Gets the id of this Lamp.  # noqa: E501

        Unique id of the lamp  # noqa: E501

        :return: The id of this Lamp.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Lamp.

        Unique id of the lamp  # noqa: E501

        :param id: The id of this Lamp.  # noqa: E501
        :type id: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Lamp.  # noqa: E501

        Name of the lamp  # noqa: E501

        :return: The name of this Lamp.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Lamp.

        Name of the lamp  # noqa: E501

        :param name: The name of this Lamp.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def channel(self):
        """Gets the channel of this Lamp.  # noqa: E501


        :return: The channel of this Lamp.  # noqa: E501
        :rtype: int
        """
        return self._channel

    @channel.setter
    def channel(self, channel):
        """Sets the channel of this Lamp.


        :param channel: The channel of this Lamp.  # noqa: E501
        :type channel: int
        """

        self._channel = channel

    @property
    def parent_group_ids(self):
        """Gets the parent_group_ids of this Lamp.  # noqa: E501

        Ids of parent groups of this lamp  # noqa: E501

        :return: The parent_group_ids of this Lamp.  # noqa: E501
        :rtype: list[str]
        """
        return self._parent_group_ids

    @parent_group_ids.setter
    def parent_group_ids(self, parent_group_ids):
        """Sets the parent_group_ids of this Lamp.

        Ids of parent groups of this lamp  # noqa: E501

        :param parent_group_ids: The parent_group_ids of this Lamp.  # noqa: E501
        :type parent_group_ids: list[str]
        """

        self._parent_group_ids = parent_group_ids

    @property
    def serial_number(self):
        """Gets the serial_number of this Lamp.  # noqa: E501

        serial number of the lamp  # noqa: E501

        :return: The serial_number of this Lamp.  # noqa: E501
        :rtype: str
        """
        return self._serial_number

    @serial_number.setter
    def serial_number(self, serial_number):
        """Sets the serial_number of this Lamp.

        serial number of the lamp  # noqa: E501

        :param serial_number: The serial_number of this Lamp.  # noqa: E501
        :type serial_number: str
        """

        self._serial_number = serial_number

    @property
    def number_of_subchannels(self):
        """Gets the number_of_subchannels of this Lamp.  # noqa: E501


        :return: The number_of_subchannels of this Lamp.  # noqa: E501
        :rtype: int
        """
        return self._number_of_subchannels

    @number_of_subchannels.setter
    def number_of_subchannels(self, number_of_subchannels):
        """Sets the number_of_subchannels of this Lamp.


        :param number_of_subchannels: The number_of_subchannels of this Lamp.  # noqa: E501
        :type number_of_subchannels: int
        """

        self._number_of_subchannels = number_of_subchannels

    @property
    def logical_address(self):
        """Gets the logical_address of this Lamp.  # noqa: E501


        :return: The logical_address of this Lamp.  # noqa: E501
        :rtype: int
        """
        return self._logical_address

    @logical_address.setter
    def logical_address(self, logical_address):
        """Sets the logical_address of this Lamp.


        :param logical_address: The logical_address of this Lamp.  # noqa: E501
        :type logical_address: int
        """

        self._logical_address = logical_address

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Lamp):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Lamp):
            return True

        return self.to_dict() != other.to_dict()



from aiohttp import ClientWebSocketResponse, WSMsgType, ClientSession
from aiohttp.client_exceptions import ClientConnectorError
from asyncio import AbstractEventLoop, iscoroutinefunction, CancelledError, TimeoutError
from aioketraapi import KeypadOperationsApi, GroupOperationsApi, ApiClient, Configuration, HubOperationsApi
from aioketraapi import WebsocketV2Notification
from aioketraapi.keypad import Keypad
from aioketraapi.group import Group
from aioketraapi.exceptions import OpenApiException

WEBSOCKET_HEARTBEAT_INTERVAL = 60.0

class N4HubWebSocketConnectionError(BaseException):
    pass

class N4HubWebSocketConnection:
    def __init__(self, api_client: ApiClient):
        self.ws = None
        self.api_client = api_client

    async def connect(self, ws_url, oauth_token):
        self.ws = await self.api_client.rest_client.client_session.ws_connect(ws_url,
                                                                              heartbeat=WEBSOCKET_HEARTBEAT_INTERVAL,
                                                                              autoping=True,
                                                                              headers={"Connection": "Upgrade"})
        # from https://s3.amazonaws.com/ketra-software/KetraMobileAPI/v1/KetraN4WebsocketProtocol.pdf
        msg = [0x06, 0x00, 0x0F]
        msg.append((len(oauth_token) >> 8) & 0xFF)
        msg.append(len(oauth_token) & 0xFF)
        msg.extend([b for b in oauth_token.encode('ascii')])
        await self.ws.send_bytes(bytearray(msg))

    async def close(self):
        assert self.ws is not None
        await self.ws.close()

    def __aiter__(self) -> "N4HubWebSocketConnection":
        assert self.ws is not None
        return self

    async def __anext__(self) -> WebsocketV2Notification:
        assert self.ws is not None
        while True:
            recv_msg = await self.ws.receive(timeout=WEBSOCKET_HEARTBEAT_INTERVAL*3)
            if recv_msg.type == WSMsgType.TEXT:
                model = self.api_client.deserialize(recv_msg, WebsocketV2Notification)
                return model
            elif recv_msg.type in (WSMsgType.CLOSE, WSMsgType.CLOSING, WSMsgType.CLOSED):
                raise StopAsyncIteration  # NOQA
            else:
                # ignore this message and go back and wait for a new one
                pass



class N4Hub():

    def __init__(self, internal_ip: str, serial_number: str, installation_id: str, url_base :str, oauth_token :str,
                 loop :AbstractEventLoop=None):
        self.internal_ip = internal_ip
        self.serial_number = serial_number
        self.installation_id = installation_id
        self.url_base = url_base
        ws_url_tmp, _, _ = url_base.rpartition('/')
        self.ws_url = f"{ws_url_tmp.replace('https', 'wss')}/notifications"
        self.oauth_token = oauth_token
        self.is_local = "api.goketra.com" not in url_base
        self.client_config = Configuration()
        self.client_config.host = f"{self.url_base}/api/v1"
        self.client_config.username = "na"
        self.client_config.password = self.oauth_token
        self.client_config.loop = loop
        if self.is_local:
            self.client_config.verify_ssl = False
        self.connected_websocket = None


    @classmethod
    async def get_hub(cls, installation_id, oauth_token, use_cloud = False, loop=None):
        """
        Returns a hub for the given installation

        :param installation_id:  installation id of Design Studio installation
        :param oauth_token:      authentication token for user account
        :param use_cloud:        False: connect to hub over local network
                                 True:  connect to hub over cloud connection
        :param loop:             optional event loop to use for async operations
        :return:                 N4Hub object
        """
        query_url = f"https://my.goketra.com/api/n4/v1/query?installation_id={installation_id}"
        async with ClientSession(loop=loop) as session:
            async with session.get(query_url) as response:
                if response.status == 200:
                    hub_query_resp = await response.json()
                    if hub_query_resp['success'] == 'true' and len(hub_query_resp['content']) > 0:
                        for hub_query_entry in hub_query_resp['content']:
                            internal_ip = hub_query_entry['internal_ip']
                            serial_number = hub_query_entry['serial_number']
                            if use_cloud:
                                url = f"https://api.goketra.com/{installation_id}/{serial_number}/webAPI"
                                hub = N4Hub(internal_ip, serial_number, installation_id, url, oauth_token, loop)
                            else:
                                url = f"https://{internal_ip}/ketra.cgi"
                                hub = N4Hub(internal_ip, serial_number, installation_id, url, oauth_token, loop)
                            if await hub.test_connectivity():
                                return hub
            return None

    def create_client_session(self):
        """
        Creates and returns an ApiClient session
        """
        return ApiClient(configuration=self.client_config)


    async def test_connectivity(self, timeout=5):
        async with self.create_client_session() as api_client:
            hub_api = HubOperationsApi(api_client)
            try:
                _ = await hub_api.hub_get(_request_timeout=timeout)
                return True
            except (TimeoutError, OpenApiException, ClientConnectorError):
                return False



    async def get_keypads(self, **kwargs):
        """
        Gets the keypads from the installation
        :param kwargs: optional arguments
                       includeall (defaults to false): include cascaded keypads
                       nobuttons (defaults to false): return only keypads, not buttons
        :return: list of Keypad objects
        """
        async with self.create_client_session() as api_client:
            kp_api = KeypadOperationsApi(api_client)
            response_model = await kp_api.keypads_get(**kwargs)
            return [Keypad(keypad, self) for keypad in response_model.content]

    async def get_groups(self, include_internal=False, **kwargs):
        """
        Gets the groups in the installation
        :param include_internal: include internally-generated master groups (begin with Internal_MasterGroup)
        :param kwargs: optional arguments
                       name:  return group matching provided name
        :return: list of Group objects
        """
        async with self.create_client_session() as api_client:
            kp_api = GroupOperationsApi(api_client)
            response_model = await kp_api.groups_get(**kwargs)
            return [Group(group, self) for group in response_model.content if include_internal or 'Internal_MasterGroup' not in group.name]

    async def __create_websocket_connection(self, api_client):
        connection = N4HubWebSocketConnection(api_client)
        await connection.connect(self.ws_url, self.oauth_token)
        return connection

    async def websocket_notifications(self) -> WebsocketV2Notification:
        async with self.create_client_session() as api_client:
            ws = await self.__create_websocket_connection(api_client)
            async for notification in ws:
                yield notification


    async def register_websocket_callback(self, notify_callback):
        """
        Connects to websocket server of hub (or cloud proxy) in order to receive notifications such as
        button press notifications or group state changes through a callback method.
        Runs indefinitely until disconnect websocket is called
        :param notify_callback:   callback function or coroutine to receive notifications.
               Should take one parameter of type WebsocketV2Notification
        """
        async with self.create_client_session() as api_client:
            try:
                ws = await self.__create_websocket_connection(api_client)
                self.connected_websocket = ws
                async for notification in ws:
                    if iscoroutinefunction(notify_callback):
                        await notify_callback(notification)
                    else:
                        notify_callback(notification)
            except (ClientConnectorError, CancelledError, TimeoutError):
                raise N4HubWebSocketConnectionError()
            finally:
                self.connected_websocket = None


    async def disconnect_websocket_callback(self):
        """
        Closes a websocket connection established with register_websocket_callback
        :return:
        """
        if self.connected_websocket is not None:
            await self.connected_websocket.close()
            self.connected_websocket = None









